/**************************************************
// datasource.js
// Implementation file for RDF DataSource
// 
// Description: Handles the data and tree structure
// Author: Aurelien
**************************************************/


var CLRDF = {

    data : null,
    file : null,

    init : function()
    {
        try {
            this.file = CLcommon.getCopyLinkerDir().clone();
            this.file.append("copylinker.rdf");
            if ( !this.file.exists() ) {
                this.file.create(this.file.NORMAL_FILE_TYPE, 0666);
                var myPath = CLservice.IO.newFileURI(this.file).spec;
                this.data = CLservice.RDF.GetDataSource(myPath);
                this.createEmptySeq("urn:copylinker:root");
                this.flush();
            }
            else {
                myPath = CLservice.IO.newFileURI(this.file).spec;
                this.data = CLservice.RDF.GetDataSource(myPath);
            }
        }
        catch(ex) {
            dump("Copylinker ERROR: Failed to initialize datasource.\n\n" 
                    + ex);
        }
    },

    flush : function()
    {
        this.data.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource)
                    .Flush();
    },

    unregister : function()
    {
        CLservice.RDF.UnregisterDataSource(this.data);
    },


    sanitize : function(aVal)
    {
        return aVal.match(/^(<|>|&)/) ? (" " + aVal) : aVal;
    },

    addItem : function(aCopyItem, aPasteItem)
    {
        /* test if the there is already a li*/
        var copySourceExists = this.hasValue(
                         CLservice.RDF.GetResource("urn:copylinker:item" + aCopyItem.id),
                         CLservice.RDF.GetResource(NS_COPYLINKER + "id"));

        var pasteSourceExists = this.hasValue(
                         CLservice.RDF.GetResource("urn:copylinker:item" + aPasteItem.id),
                         CLservice.RDF.GetResource(NS_COPYLINKER + "id"));

        try {
            if (!copySourceExists) {
              /* Creation of the source node from the page of the copy  */
              var copyRes = this.createNode (aCopyItem);
            }
          
            if (!pasteSourceExists) {
              /* Creation of the source node from the page of the paste  */
               var pasteRes = this.createNode (aPasteItem);
            }
 
            /* Add site to copy node */
            this.addCopyNode(aCopyItem, aPasteItem);

            /* Add site to paste node */
            this.addPasteNode(aCopyItem, aPasteItem);

            /* Add new sites to root node */
            try {
                CLservice.RDFC.Init(this.data, CLservice.RDF.GetResource("urn:copylinker:root"));
            } catch(ex) {
                  dump("Copylinker ERROR: Failed to initialize root container.\n"
                       + "this.data : " + this.data
                       + "\nresource : " + CLservice.RDF.GetResource("urn:copylinker:root")
                       + "\npathfile : " + CLservice.IO.newFileURI(this.file).spec 
                       + "\n" +  ex + "\n");
                  return false;
            }


            if (!copySourceExists) CLservice.RDFC.AppendElement(copyRes);
            else {
              //dump(aCopyItem.id + ' already exists');
            }
                                
            if (!pasteSourceExists) CLservice.RDFC.AppendElement(pasteRes);
            else {
              //dump(aPasteItem.id + ' already exists');
            }
            
            this.flush();
            
            //return copyRes + pasteRes;
        }
        catch(ex) {
            dump("copylinker ERROR: Failed to add element to datasource.\n\n" + ex);
        }
    },

    createNode : function (aItem) {
      try {
        var node = CLservice.RDF.GetResource("urn:copylinker:item" + aItem.id);
        this.data.Assert(node, 
                       CLservice.RDF.GetResource(NS_COPYLINKER + "id"),
                       CLservice.RDF.GetLiteral(aItem.id),
                       true);
        this.data.Assert(node,
                       CLservice.RDF.GetResource(NS_COPYLINKER + "title"),
                       CLservice.RDF.GetLiteral(aItem.title),
                       true);
        this.data.Assert(node,
                       CLservice.RDF.GetResource(NS_COPYLINKER + "comment"),
                       CLservice.RDF.GetLiteral(aItem.comment),
                       true);
        return node;
      }
      catch(ex) {
          dump("copylinker ERROR: Failed to create node.\n" +
                "urn:copylinker:item" + aItem.id + "\n" + ex);
      }
    },

    /*
     * Creation of the link between the copied page and the pasted page
     */
    addCopyNode : function (copySource, pasteSource) 
    {
        try {
            var copyRessource = CLservice.RDF.GetResource("urn:copylinker:copy" + copySource.id);
            var aSeqCopy = CLservice.RDFCU.MakeSeq(this.data, copyRessource);
                        
            var copyRes = CLservice.RDF.GetResource("urn:copylinker:pasteitem" + pasteSource.id);
            var index = aSeqCopy.IndexOf(copyRes);
            //aSeqCopy.RemoveElement(copyRes,true);

            var def = this.data.GetTarget(copyRes,
                                  CLservice.RDF.GetResource(NS_COPYLINKER + "def"),
                                  true); 
            if (def == null) {
                this.data.Assert(copyRes,
                                CLservice.RDF.GetResource(NS_COPYLINKER + "def"),
                                CLservice.RDF.GetResource("urn:copylinker:item" + pasteSource.id),
                                true);

            }
                                                     
            var nbCopyVisit = this.data.GetTarget(copyRes,
                                      CLservice.RDF.GetResource(NS_COPYLINKER + "visitNb"),
                                      true);
             
            // Update of visitNb, add 1 each time there is a copy from the page
            if (null == nbCopyVisit ) {
                this.data.Assert(copyRes,
                             CLservice.RDF.GetResource(NS_COPYLINKER + "visitNb"),
                             CLservice.RDF.GetIntLiteral(copySource.visitNb),
                             true);
            }
            else {
               if (nbCopyVisit instanceof Components.interfaces.nsIRDFInt) {
                   var nbCopyVis = nbCopyVisit.Value + 1; //CLservice.RDF.GetIntLiteral(nbCopyVisit).Value;
                   //dump('nbCopyVisit + 1 : ' + nbCopyVis + '\n');
               }
               this.data.Change(copyRes,
                                 CLservice.RDF.GetResource(NS_COPYLINKER + "visitNb"),
                                 CLservice.RDF.GetIntLiteral(nbCopyVisit.Value),
                                 CLservice.RDF.GetIntLiteral(nbCopyVis));
            }
            
            // Update of visitDate with the most recent visit date
            var date = this.data.GetTarget(copyRes,
                                      CLservice.RDF.GetResource(NS_COPYLINKER + "visitDate"),
                                      true);
            if (null == date ) {
                  this.data.Assert(copyRes,
                                 CLservice.RDF.GetResource(NS_COPYLINKER + "visitDate"),
                                 CLservice.RDF.GetLiteral(pasteSource.visitDate),
                                 true);
            }
            else {
                this.data.Change(copyRes,
                                 CLservice.RDF.GetResource(NS_COPYLINKER + "visitDate"),
                                 date,
                                 CLservice.RDF.GetLiteral(pasteSource.visitDate) );
            }
            
            if ( index == -1) {
                aSeqCopy.AppendElement(copyRes);
            }
            else {
                aSeqCopy.InsertElementAt(copyRes,index,false);
            }
        } catch(ex) {
            dump("Copylinker ERROR: Failed to initialize copy container.\n\n" + ex);
        }
    },
    
    /*
     * Creation of the link between the pasted page and the copied page 
     */
    addPasteNode : function (copySource, pasteSource) 
    {
        try {
            var pasteRessource = CLservice.RDF.GetResource("urn:copylinker:paste" + pasteSource.id);
            var aSeqPaste = CLservice.RDFCU.MakeSeq(this.data, pasteRessource);

            var pasteRes = CLservice.RDF.GetResource("urn:copylinker:copyitem" + copySource.id);
              //aSeqPaste.RemoveElement(pasteRes, true);
            var index = aSeqPaste.IndexOf(pasteRes);

            var def = this.data.GetTarget(pasteRes,
                                  CLservice.RDF.GetResource(NS_COPYLINKER + "def"),
                                  true); 
            if (def == null) {
                this.data.Assert(pasteRes,
                               CLservice.RDF.GetResource(NS_COPYLINKER + "def"),
                                CLservice.RDF.GetResource("urn:copylinker:item" + copySource.id),
                                true);
            }
            
            var nbPasteVisit = this.data.GetTarget(pasteRes,
                                      CLservice.RDF.GetResource(NS_COPYLINKER + "visitNb"),
                                      true);

            if (null == nbPasteVisit ) {
                this.data.Assert(pasteRes,
                             CLservice.RDF.GetResource(NS_COPYLINKER + "visitNb"),
                             CLservice.RDF.GetIntLiteral(copySource.visitNb),
                             true);
            }
            else {
               if (nbPasteVisit instanceof Components.interfaces.nsIRDFInt) {
                   var nbPasteVis = nbPasteVisit.Value + 1; 
                   //dump('nbPasteVisit + 1 : ' + nbPasteVis + '\n');
               }
               this.data.Change(pasteRes,
                                 CLservice.RDF.GetResource(NS_COPYLINKER + "visitNb"),
                                 CLservice.RDF.GetIntLiteral(nbPasteVisit.Value),
                                 CLservice.RDF.GetIntLiteral(nbPasteVis));
            }
            var date = this.data.GetTarget(pasteRes,
                                      CLservice.RDF.GetResource(NS_COPYLINKER + "visitDate"),
                                      true);
            if (null == date ) {
                  this.data.Assert(pasteRes,
                                 CLservice.RDF.GetResource(NS_COPYLINKER + "visitDate"),
                                 CLservice.RDF.GetLiteral(copySource.visitDate),
                                 true);
            }
            else {
                this.data.Change(pasteRes,
                                 CLservice.RDF.GetResource(NS_COPYLINKER + "visitDate"),
                                 date,
                                 CLservice.RDF.GetLiteral(copySource.visitDate) );
            }

            if ( index == -1) {
                aSeqPaste.AppendElement(pasteRes);
            }
            else {
                aSeqPaste.InsertElementAt(pasteRes,index,false);
            }
        } catch(ex) {
            dump("Copylinker ERROR: Failed to initialize paste container.\n\n" + ex);
        }
    },


    moveItem : function(curRes, curPar, tarPar, tarRelIdx)
    {
        try {
            CLservice.RDFC.Init(this.data, curPar);
            CLservice.RDFC.RemoveElement(curRes, true);
        } catch(ex) {
            dump("copylinker ERROR: Failed to move element at datasource (1).\n\n" + ex);
            return;
        }
        try {
            CLservice.RDFC.Init(this.data, tarPar);
            if ( tarRelIdx > 0 ) {
                CLservice.RDFC.InsertElementAt(curRes, tarRelIdx, true);
            } else {
                CLservice.RDFC.AppendElement(curRes);
            }
        }
        catch(ex) {
            dump("copylinker ERROR: Failed to move element at datasource (2).\n\n" + ex);
            CLservice.RDFC.Init(this.data, CLservice.RDF.GetResource("urn:copylinker:root"));
            CLservice.RDFC.AppendElement(curRes, true);
        }
    },

    updateItem : function(aRes, aProp, newVal)
    {
//        newVal = this.sanitize(newVal);
        try {
            aProp = CLservice.RDF.GetResource(NS_COPYLINKER + aProp);
            var oldVal = this.data.GetTarget(aRes, aProp, true);
            oldVal = oldVal.QueryInterface(Components.interfaces.nsIRDFLiteral);
            newVal = CLservice.RDF.GetLiteral(newVal);
            this.data.Change(aRes, aProp, oldVal, newVal);
        }
        catch(ex) {
            dump("copylinker ERROR: Failed to update element of datasource.\n" + ex);
        }
    },

    hasValue : function (subject, predicate) {
        try { 
            var val = this.data.GetTarget(subject, predicate, true);
            if (val != null) var sourceExists = true;        
            else sourceExists = false;
        }
        catch (ex) {
            sourceExists = false;
        }
        //dump(subject + ' ' + predicate + ' has no value\n');
        return (sourceExists);
    },

    createEmptySeq : function(aResName)
    {
        CLservice.RDFCU.MakeSeq(this.data, CLservice.RDF.GetResource(aResName));
    },


    addMenuItem : function(aCopyItem, aPasteItem)
    {
         var copySource = CLservice.RDF.GetResource("urn:copylinker:item" + aCopyItem.id);
         var pasteSource = CLservice.RDF.GetResource("urn:copylinker:item" + aPasteItem.id);
        
            var copyRessource = CLservice.RDF.GetResource("urn:copylinker:copy" + aCopyItem.id);
            var aSeqCopy = CLservice.RDFCU.MakeSeq(this.data, copyRessource);
                        
            var copyRes = CLservice.RDF.GetResource("urn:copylinker:pasteitem" + aPasteItem.id);
            var index = aSeqCopy.IndexOf(copyRes);

            var nbItemUsed = this.data.GetTarget(copyRes,
                                      CLservice.RDF.GetResource(NS_COPYLINKER + "nbItemUsed"),
                                      true);
             
            // Update of visitNb, add 1 each time there is a copy from the page
            if (null == nbItemUsed ) {
                this.data.Assert(copyRes,
                             CLservice.RDF.GetResource(NS_COPYLINKER + "nbItemUsed"),
                             CLservice.RDF.GetIntLiteral(1),
                             true);
            }
            else {
               if (nbItemUsed instanceof Components.interfaces.nsIRDFInt) {
                   var nbItemUsedPlus = nbItemUsed.Value + 1; 
               }
               this.data.Change(copyRes,
                                 CLservice.RDF.GetResource(NS_COPYLINKER + "nbItemUsed"),
                                 CLservice.RDF.GetIntLiteral(nbItemUsed.Value),
                                 CLservice.RDF.GetIntLiteral(nbItemUsedPlus));
            }


            this.flush();
            return nbItemUsed ? nbItemUsedPlus : 1 ;
    }
};
